set -x

version=$1

DIRNAME=`dirname "$0"`
if [ "$DIRNAME" = "." ]; then
  DIRNAME="$PWD"
fi

mkdir -p "$DIRNAME/target/tomee/"
mkdir -p "$DIRNAME/target/tomcat/"

pushd "$DIRNAME" > /dev/null

for d in *; do
  bn=$(basename $d)
  if [ "$bn" = "catalina" ]; then
    bn="tomee-catalina"
  fi
  if [ -d "$d/src/main/config/patch" ]; then
    curl https://repo1.maven.org/maven2/org/apache/tomee/$bn/$version/$bn-$version-sources.jar > $DIRNAME/target/tomee/$bn-$version-sources.jar
    pushd "$d/src/main/config/patch" > /dev/null
    for f in *; do
      bn2=$(basename $f)
      fn=$(echo $bn2 | sed 's|\.|/|g').java
      pushd "../../java" > /dev/null
      if [ "$2" != "" ]; then
        cp $fn $fn.new
        jar xf $DIRNAME/target/tomee/$bn-$version-sources.jar $fn
        dos2unix $fn
        diff -u $fn $fn.new > ../config/patch/$f
        sed -i~ 's/^\(+++\|---\)\(.*\)\t[0-9]\+-[0-9]\+-[0-9]\+ [0-9]\+:[0-9]\+:[0-9]\+\.[0-9]\+ .[0-9]\+/\1\2\t2021-01-01 00:00:00.000000000 +0000/g' ../config/patch/$f
        rm ../config/patch/$f~
        rm $fn.new     
        git checkout HEAD -- $fn
      else
        cp $fn $fn.back
        jar xf $DIRNAME/target/tomee/$bn-$version-sources.jar $fn
        dos2unix $fn
        patch $fn ../config/patch/$f
        rm $fn.back
      fi
      popd > /dev/null
    done
    popd > /dev/null
  fi
done

popd > /dev/null

if [ ! -f "$DIRNAME/target/tomee/apache-tomee-${version}-webprofile.zip" ]; then
  curl https://repo.maven.apache.org/maven2/org/apache/tomee/apache-tomee/${version}/apache-tomee-${version}-webprofile.zip > "$DIRNAME/target/tomee/apache-tomee-${version}-webprofile.zip"
fi
pushd "$DIRNAME/target/tomee" > /dev/null
rm -rf apache-tomee-webprofile-${version}
unzip apache-tomee-${version}-webprofile.zip
pushd apache-tomee-webprofile-${version} > /dev/null

tomcat_version=$(grep "Apache Tomcat Version" RELEASE-NOTES | sed 's/.*\([0-9]\+\.[0-9]\+\.[0-9]\+\).*/\1/')

tomee_javaee_api_version=$(ls lib/javaee-api-*.jar | sed 's|lib/javaee-api-\(.*\)\.jar|\1|')

rm conf/tomee.xml
sed -i~ 's/autoDeploy="true"/autoDeploy="true" deployXML="true"/' conf/server.xml
rm conf/server.xml~
sed -i~ 's/autoDeploy="true"/autoDeploy="true" deployXML="true"/' conf/server.xml.original
rm conf/server.xml.original~


rm -rf "$DIRNAME/installer/src/main/resources/org/apache/tomee/vestige/installer/conf"
mv "$DIRNAME/target/tomee/apache-tomee-webprofile-${version}/conf" "$DIRNAME/installer/src/main/resources/org/apache/tomee/vestige/installer"
find . -depth 1 -type f -exec mv \{\} "$DIRNAME/installer/src/main/resources/org/apache/tomee/vestige/installer/root/" \;


popd > /dev/null
popd > /dev/null


MAIN_VERSION=$(echo ${tomcat_version} | cut -d. -f1)

if [ ! -f "$DIRNAME/target/tomcat/apache-tomcat-${tomcat_version}.zip" ]; then
  curl https://archive.apache.org/dist/tomcat/tomcat-$MAIN_VERSION/v${tomcat_version}/bin/apache-tomcat-${tomcat_version}.zip > "$DIRNAME/target/tomcat/apache-tomcat-${tomcat_version}.zip"
fi
rm -rf "$DIRNAME/target/tomcat/apache-tomcat-${tomcat_version}"
mkdir -p "$DIRNAME/target/tomcat/apache-tomcat-${tomcat_version}"
pushd "$DIRNAME/target/tomcat" > /dev/null
unzip apache-tomcat-${tomcat_version}.zip
pushd apache-tomcat-${tomcat_version} > /dev/null


rm -rf bin lib logs temp


rm "$DIRNAME/installer/src/main/resources/org/apache/tomee/vestige/installer/root_excludes.txt"

for cf in $(find . -depth 1 -type f  | sed 's/^\..//'); do
  if [ -f "$DIRNAME/installer/src/main/resources/org/apache/tomee/vestige/installer/root/$cf" ]; then
    # compare
    dos2unix "$DIRNAME/installer/src/main/resources/org/apache/tomee/vestige/installer/root/$cf"
    dos2unix "$cf"
    if cmp "$DIRNAME/installer/src/main/resources/org/apache/tomee/vestige/installer/root/$cf" "$cf"; then
      # same tomcat can install
      rm "$DIRNAME/installer/src/main/resources/org/apache/tomee/vestige/installer/root/$cf"
    else
      # different must be excluded
      echo $cf >> "$DIRNAME/installer/src/main/resources/org/apache/tomee/vestige/installer/root_excludes.txt"
    fi
  fi
done


rm "$DIRNAME/installer/src/main/resources/org/apache/tomee/vestige/installer/conf_excludes.txt"
for cf in $(find conf -type f); do
  if [ -f "$DIRNAME/installer/src/main/resources/org/apache/tomee/vestige/installer/$cf" ]; then
    # compare
    dos2unix "$DIRNAME/installer/src/main/resources/org/apache/tomee/vestige/installer/$cf"
    dos2unix "$cf"
    if cmp "$DIRNAME/installer/src/main/resources/org/apache/tomee/vestige/installer/$cf" "$cf"; then
      # same tomcat can install
      rm "$DIRNAME/installer/src/main/resources/org/apache/tomee/vestige/installer/$cf"
    else
      # different must be excluded
      echo $cf >> "$DIRNAME/installer/src/main/resources/org/apache/tomee/vestige/installer/conf_excludes.txt"
    fi
  fi
done


popd > /dev/null
popd > /dev/null




pushd "$DIRNAME/installer/src/main/resources/org/apache/tomee/vestige/installer"
find conf -type f > conf.txt
pushd root
find . -type f  | sed 's/^\..//' > ../root.txt
popd
popd

mvn versions:set -DnewVersion=$version.1-SNAPSHOT -DgenerateBackupPoms=false

sed -i~ 's/<tomee.version>.*<\/tomee.version>/<tomee.version>'${version}'<\/tomee.version>/' pom.xml
sed -i~ 's/<tomee.javaee-api.version>.*<\/tomee.javaee-api.version>/<tomee.javaee-api.version>'${tomee_javaee_api_version}'<\/tomee.javaee-api.version>/' pom.xml
sed -i~ 's/<tomcat.version>.*<\/tomcat.version>/<tomcat.version>'${tomcat_version}'<\/tomcat.version>/' pom.xml
rm pom.xml~

sed -i~ 's/version: .*/version: '${version}'/' README.md
rm README.md~

